/*
 * Test removal operations.
 *
 * Copyright (C) 2020, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <fsclient/client.h>
#include <systypes/fcntl.h>



static void read_from_file(file_t *file)
{
  char buf[20];
  offset_t nread;

  client_seek(file, 0, SEEK_SET);

  printf("Reading...\n");

  nread = client_read(file, buf, 20);

  printf("Read %ld bytes...\n", nread);
  fwrite(buf, sizeof(char), nread, stdout);
  fputs("\n", stdout);
}

static void write_to_file(file_t *file, const char *buf)
{
  offset_t nwritten;

  client_seek(file, 0, SEEK_SET);

  printf("Writing...\n");

  nwritten = client_write(file, buf, strlen(buf));

  printf("Wrote %ld bytes: ", nwritten);
  fwrite(buf, sizeof(char), nwritten, stdout);
  fputs("\n", stdout);
}

int main(int argc, char *argv[])
{
  if (argc < 2)
  {
    printf("Need a file to remove.\n");
    return 1;
  }

  char *filename = argv[1];

  printf("Creating %s...\n", filename);

  file_t *file = client_open(filename, O_WRONLY | O_CREAT);

  write_to_file(file, "An existing file.");

  client_close(file);

  printf("Opening %s...\n", filename);

  file = client_open(filename, O_RDONLY);

  if (file == NULL)
  {
    printf("Could not open file: %s\n", filename);
    return 1;
  }

  /* Read from the file, then remove the file, then read again. */

  read_from_file(file);

  printf("Removing...\n");

  long err = client_remove(filename);

  if (err)
  {
    printf("Could not remove file: %s\n", filename);
    return 1;
  }

  printf("Reading again...\n");

  read_from_file(file);

  /* Open the file again. */

  file_t *file_new = client_open(filename, O_RDONLY);

  if (file_new != NULL)
  {
    printf("File should be absent: %s\n", filename);
    return 1;
  }

  file_new = client_open(filename, O_RDWR | O_CREAT);

  if (file_new == NULL)
  {
    printf("File should be present: %s\n", filename);
    return 1;
  }

  write_to_file(file_new, "New file!");

  /* Read from the initial file and new file. */

  read_from_file(file);
  read_from_file(file_new);

  /* Close the files. */

  client_close(file);
  client_close(file_new);

  printf("End of tests.\n");

  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
